#!/bin/bash

# This script installs a mandriva Cooker under a certain root directory
DIR=$1
VHD=$2
ARCH=$3
DISTRIB=$4/${ARCH}/
FS=$5
if [ -z ${DIR} ]; then
	echo "Cannot install in non-set directory."
	exit 1
fi

if [ -z ${VHD} ]; then
	echo "No virtual hard drive was set."
	exit 1
fi

if [ ! -d ${DIR} ]; then
	echo "Cannot access new root directory (${DIR})"
	exit 1
fi

if [ ! -f ${VHD} ]; then
	echo "Cannot access image file (${VHD})"
	exit 1
fi

. /usr/lib/mdvvhdprov/common.sh

vol_id()
{
	DEV=$2
	RES=$(blkid ${DEV} |sed -ne 's/.*UUID="\([^\"]*\)" .*/\1/p')
	eval "$1=$RES"
	echo "Identified ${DEV} as ${RES}"
}

generate_etc_fstab()
{
	echo "Creating ${DIR}/etc/fstab"
	vol_id UUID_BOOT /dev/${BOOT_LOOP}
	vol_id UUID_ROOT /dev/${ROOT_LOOP}
	vol_id UUID_SWAP /dev/${SWAP_LOOP}
	cat >${DIR}/etc/fstab <<EOF
tmpfs			/dev/shm	tmpfs	defaults        0 0
devpts			/dev/pts	devpts	gid=5,mode=620  0 0
sysfs			/sys		sysfs	defaults        0 0
proc			/proc		proc	defaults        0 0
UUID=$UUID_ROOT		/		${FS}	defaults,errors=remount-ro	0	2
UUID=$UUID_SWAP		none		swap	sw	0	0
UUID=$UUID_BOOT		/boot		${BOOT_FS}	defaults,errors=remount-ro	0	0
EOF
	if [ $? -ne 0 ]; then
		echo "Error while populating ${DIR}/etc/fstab"
		exit 1
	else
		echo "${DIR}/etc/fstab created successfully."
	fi
}

generate_boot_grub_menu_lst()
{
	vol_id UUID_ROOT /dev/${ROOT_LOOP}
	mkdir -p ${DIR}/boot/grub/
	cat >${DIR}/boot/grub/menu.lst <<EOF
timeout 1
default 0

title linux
	kernel (hd0,0)/vmlinuz-desktop root=UUID=${UUID_ROOT} nosplash
	initrd (hd0,0)/initrd-desktop.img
EOF

	if [ $? -ne 0 ]; then
		echo "Error while populating ${DIR}/boot/grub/menu.lst"
		exit 1
	else
		echo "${DIR}/boot/grub/menu.lst created successfully."
	fi
}

install_grub_vhd()
{
	GRUB_BATCH=$(mktemp)
	for img in ${DIR}/lib/grub/*/*; do
		echo "Copying ${img} to ${DIR}/boot/grub/"
		cp ${img} ${DIR}/boot/grub/
	done;

	sync

	cat >${GRUB_BATCH} <<EOF
device (hd0) ${VHD}
root (hd0,0)
setup (hd0)
EOF

	sync

	grub --batch --no-floppy --device-map=/dev/null < ${GRUB_BATCH}
	if [ $? -ne 0 ]; then
		echo "Error while installing grub."
		exit 1
	else
		echo "GRUB successfully installed !"
	fi

	sync
}

generate_initrd()
{
	echo "Generating initrd ..."
	for kernel in ${DIR}/lib/modules/*; do
		KERNEL=$(echo $kernel|sed -ne 's/.*modules\/\([^\"]*\)/\1/p')
		INITRD=initrd-${KERNEL}.img
		echo "Calculating dependencies ..."
		chroot ${DIR} depmod ${KERNEL} -a
		echo "Calling initrd generator ..."
		chroot ${DIR} mkinitrd -f \
			--with jbd \
			--with libata \
			--with ahci \
			--with ata_piix \
			--with crc-t10dif \
			--with scsi_mod \
			--with sd_mod \
			/boot/${INITRD} ${KERNEL}
	done
	echo "Generated initrd."
}

generate_network_cfg()
{
	cat >${DIR}/etc/sysconfig/network <<EOF
HOSTNAME=mdv-testcloud-$(uuidgen)
NETWORKING=yes
EOF

	cat >${DIR}/etc/sysconfig/network-scripts/ifcfg-eth0 <<EOF
DEVICE=eth0
BOOTPROTO=dhcp
ONBOOT=yes
DHCP_CLIENT=dhclient
PEERDNS=yes
EOF
}

add_urpmi_media()
{
	urpmi.addmedia --urpmi-root=${DIR} --distrib --mirrorlist ${DISTRIB}
}

prepare_distrib()
{
	add_urpmi_media
}

last_prepare_distrib()
{
	generate_initrd
	generate_network_cfg
	generate_etc_fstab
	generate_boot_grub_menu_lst
	install_grub_vhd
}

urpmi_install()
{
	echo "Installing ${DISTRIB} in ${DIR}"
	urpmi --auto --download-all --use-distrib ${DISTRIB} --root ${DIR} basesystem kernel-desktop-latest grub urpmi dhcp-client
}

prepare_distrib
urpmi_install
last_prepare_distrib
