#!/bin/bash

# This script creates a RAW Virtual Hard Drive and install Mandriva on it.

VHD=$1
VHDSIZE=$2
VHDFS=$3
VHDARCH=$4
VHDMIRROR=$5

# Source common variables
. /usr/lib/mdvvhdprov/common.sh

usage()
{
	echo "$0 VHD SIZE FS ARCH MIRROR"
	cat <<EOF
Where :
	- VHD 	: is the path of the VHD to create.
	- SIZE	: is the size of the VHD to create (in MB).
		/boot will be 32M, and swap will be 256M.
	- FS	: is the filesystem used for the VHD.
	- ARCH 	: is the architecture of the VM to create (e.g. i586, x86_64).
	- MIRROR: is the URL of the RPM mirror to use.

Mirrors are listed on http://api.mandriva.com/mirrors/list.php

Example :
Creating a 4G Virtual Hard Drive ext4 for x86_64 Cooker, in /tmp/mdvcooker.vhd :
$0 /tmp/mdvcooker.vhd 4096 ext4 x86_64 http://ftp.free.fr/mirrors/ftp.mandriva.com/MandrivaLinux/devel/cooker/
EOF
}

check_root()
{
	if [ $UID -ne 0 ]; then
		echo "Not running as root. Aborting."
		return 1
	else	return 0
	fi
}

check_losetup()
{
	for dev in ${LODEVICES};
	do
		# Check /dev/loopX is a block device
		DEV=/dev/$dev
		if [ ! -b ${DEV} ]; then
			echo "Not a block device: ${DEV}"
			return 1
		fi

		# Check /dev/loopX is being already used
		# losetup return 0 if it is
		# and 1 if nothing is in /dev/loopX
		LOS=$(losetup ${DEV} 2>/dev/null)
		if [ $? -eq 0 ]; then
			echo "losetup error on ${DEV}: ${LOS}"
			return 1
		fi
	done
}

check_arguments()
{
	GOOD=0
	if [ -z ${VHD} ]; then
		echo "No VHD file specified."
		GOOD=0
	else	GOOD=1
	fi

	if [ -z ${VHDSIZE} ]; then
		echo "Cannot specify empty size."
		GOOD=0
	else
		if [ ${MINVHDSIZE} -gt ${VHDSIZE} ]; then
			echo "Cannot specify size < (256+32)MB. Aborting."
			GOOD=0
		else	GOOD=1
		fi
	fi

	if [ -z ${VHDARCH} ]; then
		echo "Cannot specify empty arch."
		GOOD=0
	else
		case "${VHDARCH}" in
			i586|x86_64)
				GOOD=1
			;;
			*)
				echo "Invalid ARCH specified. Aborting."
				GOOD=0
			;;
		esac
	fi

	if [ -z ${VHDFS} ]; then
		echo "Cannot specify empty FS."
		GOOD=0
	else
		which mkfs.${VHDFS}
		if [ $? -ne 0 ]; then
			echo "Cannot use the following file system : ${VHDFS}. No mkfs.${VHDFS} found."
			GOOD=0
		else	GOOD=1
		fi
	fi

	if [ -z ${VHDMIRROR} ]; then
		echo "Cannot specify empty mirror."
		GOOD=0
	else
		aria2c ${VHDMIRROR} -o /dev/null
		GET_BASE=$?
		aria2c ${VHDMIRROR}/SRPMS/ -o /dev/null
		GET_SRPMS=$?

		if [ ${GET_BASE} -ne 0 ]; then
			echo "Specified mirror seems invalid (check1). Please verify."
			GOOD=0
		else	if [ ${GET_SRPMS} -ne 0 ]; then
				echo "Specified mirror seems invalid (check2). Please verify."
				GOOD=0
			else	GOOD=1
			fi
		fi
	fi

	if [ ${GOOD} -eq 0 ]; then
		usage
		exit 1
	fi
}

check_mountpoint()
{
	if [ ! -d ${ROOT} ]; then
		echo "The mountpoint ${ROOT} isn't accessible. Cannot continue."
		return 1
	fi

	fuser ${ROOT} 2>/dev/null 1>/dev/null
	if [ ! $? -eq 1 ]; then
		echo "Mountpoint ${ROOT} already mounted, please check."
		return 1
	fi

	return 0
}

check_vhdpath()
{
	touch ${VHD}

	if [ $? -ne 0 ]; then
		echo "The VHD ${VHD} cannot be created. Check for perms etc."
		return 1
	fi

	rm ${VHD}
	return 0
}

check_urpmi()
{
	which urpmi
	if [ $? -ne 0 ]; then
		echo "Cannot find urpmi. Aborting."
		return 1
	fi

	return 0
}

check_parted()
{
	which parted
	if [ $? -ne 0 ]; then
		echo "Cannot find parted. Aborting."
		return 1
	fi

	return 0
}

check_prerequisties()
{
	check_arguments

	check_root
	if [ $? -ne 0 ]; then
		echo "You need root rights to run this tool."
		exit 1
	fi

	check_losetup
	if [ $? -ne 0 ]; then
		echo "Please check the loopback devices availability:"
		for d in ${LODEVICES};
		do
			echo " - /dev/$d"
		done
		exit 1
	fi

	check_mountpoint
	if [ $? -ne 0 ]; then
		usage
		exit 1
	fi

	check_vhdpath
	if [ $? -ne 0 ]; then
		usage
		exit 1
	fi

	check_urpmi
	if [ $? -ne 0 ]; then
		usage
		exit 1
	fi

	check_parted
	if [ $? -ne 0 ]; then
		usage
		exit 1
	fi
}

check_prerequisties

create_vm_hdd.sh ${VHD} ${VHDSIZE} ${VHDFS}
if [ $? -ne 0 ]; then
	echo "Error while creating the VHD"
	exit 1
fi

partmount.sh ${VHDFS}
if [ $? -ne 0 ]; then
	echo "Error while mounting the filesystem. Aborting."
	exit 1
fi

echo "Virtual Hard Drive mounted. Ready for installation."

mdvbootstrap.sh ${ROOT} ${VHD} ${VHDARCH} ${VHDMIRROR} ${VHDFS}
if [ $? -ne 0 ]; then
	echo "Error while installing Cooker to the VHD"
	exit 1
fi

partumount.sh
if [ $? -ne 0 ]; then
	echo "Error while unmounting the VHD"
	exit 1
fi

unlosetup.sh
if [ $? -ne 0 ]; then
	echo "Error while cleaning the loop devices"
	exit 1
fi
